<?php

namespace Drupal\wsolution\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\wsolution\Passeplat\PassePlatComponent;
use Drupal\wsolution\Passeplat\StreamProcessor\SchemeProcessor\VCRSchemeProcessor;
use Drupal\wsolution\Vcr\WSVcr;
use Drupal\wsolution\Passeplat\EventListener\EventListenerDefinition;
use Symfony\Component\HttpFoundation\Request;
use PassePlat\Core\StreamProcessor\SchemeProcessor\Event\ProcessSchemeEvent;
use PassePlat\Core\Event\ProcessedRequestEvent;

/**
 * Class devisService.
 */
class WSolution implements WSolutionInterface {

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  protected $tempstorePrivate;

  /**
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $tempstore_private
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   */
  public function __construct(ConfigFactoryInterface $config_factory, PrivateTempStoreFactory $tempstore_private, RouteProviderInterface $route_provider, PathMatcherInterface $path_matcher) {
    // Setup object members.
    $this->configFactory = $config_factory;
    $this->tempstorePrivate = $tempstore_private->get('ws_php_vcr');
    $this->routeProvider = $route_provider;
    $this->pathMatcher = $path_matcher;
  }

  /**
   * Start listening outgoing request.
   *
   * @return bool
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function startListeningOutgoing() {
    $passeplat_settings = $this->configFactory->get('wsolution.outgoing.settings');
    $this->tempstorePrivate->set('ws_php_vcr_start', TRUE);
    // Start listening with PHP VCR.
    WSVcr::turnOn();
    WSVcr::configure()
      ->setCassettePath(DRUPAL_ROOT . $passeplat_settings->get('file_path'));
    $library = $passeplat_settings->get('library') ?? [
      'curl',
      'soap',
      'stream_wrapper',
    ];
    WSVcr::configure()->enableLibraryHooks($library);

    // Record requests and responses in cassette file.
    WSVcr::insertCassette(date('Y/m/d/H:i', time()) . '.' . $passeplat_settings->get('storage'));
    return TRUE;
  }

  /**
   * Stop listening outgoing request.
   *
   * @return bool
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function stopListeningOutgoing() {
    if ($this->tempstorePrivate->get('ws_php_vcr_start') === TRUE) {
      // To stop recording requests, eject the cassette.
      WSVcr::eject();
      // Turn off VCR to stop intercepting requests.
      WSVcr::turnOff();
      $this->tempstorePrivate->delete('ws_php_vcr_start');
      return TRUE;
    }
    return FALSE;
  }

}
